# Import Package
import pandas as pd
import RLOHE as r             # Rare Label One-Hot Encoder
import warnings

# Other Settings
warnings.filterwarnings('ignore')
pd.options.display.float_format = '{:.6f}'.format
pd.set_option('display.max_columns', 256)


# Main Method
if __name__ == '__main__':

    # Loading in Dataset
    train = pd.read_csv('https://raw.githubusercontent.com/rahulbordoloi/Rare-Label-One-Hot-Enocder/main/Data/Train_Data.csv')
    test = pd.read_csv('https://raw.githubusercontent.com/rahulbordoloi/Rare-Label-One-Hot-Enocder/main/Data/Test_Data.csv')

    # Top Labeled Entries in Both of the Sets for Analysis
    r.TopLabeledEntries(train, test, feature_name = 'department_info', threshold = 10, secondary_feature = 'cost_to_pay')

    # Rare Label One-Hot Encoder [Level Wise]
    encodedTrain, encodedTest = r.RareLabelOneHotEncoder(train, test, feature_name = 'department_info', threshold = 10,
                                                         criterion = 'level', prefix_name = 'dept', verbose = True)




# Import Libraries
import pandas as pd


# Top-Labeled Function
def TopLabeledEntries(train = None, test = None, feature_name = None, threshold = 10, criterion = 'level', secondary_feature = None, verbose = True, return_dataframe = False):

    """
    Gives out Top Labeled Entries' Analysis of two given DataFrames.
    :param train: Train Dataset.
    :param test: Test Dataset.
    :param feature_name: Feature on which encoding is to be done.
    :param threshold: Top Features Seggregator Limit.
    :param criterion: `level/volume` according to which top entries will be picked up.
    :param secondary_feature: To check amount statistics of another feature with respect to the primary feature.
    :param verbose: Variable to Control Output to the console.
    :param return_dataframe: condition for if a dataframe has to be returned or not.
    :return: None/Dataframe.

    __Reference__
    *  Level : Will be considering up top `level` threshold entries for the particular feature, and rest as `BELOW`.
    *  Amount : Will be considering up the entries above the threshold for the particular feature, and rest as `BELOW`.
    """

    # Sanity Check [for DataFrames]
    if pd.DataFrame(train).empty or pd.DataFrame(test).empty:
        raise ValueError('Either of the DataFrame is NULL')

    # Creating Copies of the DataFrames
    train_set = train.copy()
    test_set = test.copy()

    # Taking out the Counts of Entries
    try:

        counts_train = train_set.groupby([feature_name])[feature_name].count().sort_values(ascending = False)
        counts_test = test_set.groupby([feature_name])[feature_name].count().sort_values(ascending = False)

    # Stop Execution and Print Error Message if the given feature doesn't exist.
    except:
        raise ValueError(f"Feature '{feature_name}' not found in the DataFrame!")

    # Taking out the Indexes [Names] of the Entries ABOVE the Threshold

    # Check if criterion is `level`
    if criterion == 'level':

        important_entries_train = counts_train[:threshold].index.tolist()
        important_entries_test = counts_test[:threshold].index.tolist()

    # Check if criterion is `volume`
    elif criterion == 'volume':

        important_entries_train = counts_train[counts_train >= threshold].index.tolist()
        important_entries_test = counts_test[counts_test >= threshold].index.tolist()

    # If Criterion Not Found
    else:
        raise ValueError(f"Criterion type '{criterion}' not found!")

    # Taking out the Indexes [Names] of the Entries BELOW the Threshold

    # [Level Criterion]
    if criterion == 'level':

        below_threshold_train = counts_train[threshold:].index
        below_threshold_test = counts_test[threshold:].index

    # [Volume Criterion]
    elif criterion == 'volume':

        below_threshold_train = counts_train[counts_train < threshold].index
        below_threshold_test = counts_test[counts_test < threshold].index

    # Imputing Entries Below Threshold as 'BELOW' to put them into a Single Category
    train_set[feature_name].replace(below_threshold_train, 'BELOW', inplace = True)
    test_set[feature_name].replace(below_threshold_test, 'BELOW', inplace = True)

    # Grouping Entries after Imputing 'BELOW'

    impute_counts_train = train_set[feature_name].value_counts().sort_values(ascending = False)
    impute_counts_test = test_set[feature_name].value_counts().sort_values(ascending = False)

    # Printing the Result if Verbose is True
    if verbose:

        ## Entries Level Statistics
        print()
        print("*" * 16, "Entries Level Statistics", "*" * 16)
        print()
        print(f"Total Unique Entries of '{feature_name}' in Train : {len(train[feature_name].unique().tolist())}")
        print(f"Total Unique Entries of '{feature_name}' in Test : {len(test[feature_name].unique().tolist())}")
        print()

        # Check `criterion` for Output String
        output_String = f"Contribution of Top {threshold} Entries" if criterion == "level" else f"Contribution of Entries <= {threshold}"

        ## Give out Volume Statistics
        print(f"Volume {output_String} [Train] : {round(sum(counts_train[:threshold]) / len(train_set) * 100, 2)} %")
        print(f"Volume {output_String} [Test]  : {round(sum(counts_test[:threshold]) / len(test_set) * 100, 2)} %")
        print()

        # Check for the Presence of Second Feature [Amount Statistics]
        if secondary_feature:
            print(f"Amount {output_String} [Train] : {round(train_set[train_set[feature_name].isin(important_entries_train)][secondary_feature].sum() / sum(train_set[secondary_feature]) * 100, 2)} %")
            print(f"Amount {output_String} [Test]  : {round(test_set[test_set[feature_name].isin(important_entries_test)][secondary_feature].sum() / sum(test_set[secondary_feature]) * 100, 2)} %")

        # Check `criterion` for Output String
        output_String = f"Common Entries in Top {threshold} Entries" if criterion == "level" else f"Common Entries in Entries <= {threshold}"

        ## Data wise Statistics for Important Entries
        print(f"{output_String} of Both the Sets Combined : {len(set(important_entries_train) & set(important_entries_test))}")
        print(f"Entries that are in Train not in Test : {list(set(important_entries_test) - set(important_entries_train))}")
        print(f"Entries that are in Test not in Train : {list(set(important_entries_train) - set(important_entries_test))}")
        print()

        # Check `criterion` for Output String
        output_String = f"Entries Above & Equal to {threshold}" if criterion == "volume" else f"Top {threshold} Entries"

        ## Train-Test wise Final Groupby
        print("*" * 26, f"Train {output_String}", "*" * 26)
        print(impute_counts_train)
        print()
        print("*" * 26, f"Test {output_String}", "*" * 26)
        print(impute_counts_test)
        print()
        print("Note : 'BELOW' Category represents all the entries below the Threshold.")

    # Checking Condition for `return`
    if return_dataframe:
        return train_set, test_set











# Importing Libraries
import pandas as pd
import warnings
from RLOHE.TopLabeledEntries import TopLabeledEntries
from RLOHE.RareLabelOneHotEncoder import RareLabelOneHotEncoder

# Other Settings
warnings.filterwarnings('ignore')
pd.options.display.float_format = '{:.6f}'.format
pd.set_option('display.max_columns', 256)




# Import Libraries & Files
import pandas as pd
from RLOHE.TopLabeledEntries import TopLabeledEntries


# Rare-Label-One-Hot Encoder
def RareLabelOneHotEncoder(train = None, test = None, feature_name = None, threshold = 10, criterion = 'level',
                           verbose = False, prefix_name = None):

    """
    Gives out Rare Label One-Hot Encoded DataFrames according to threshold being set and it's criterion of segregation.
    :param train: Train Dataset.
    :param test: Test Dataset.
    :param feature_name: Feature on which encoding is to be done.
    :param threshold: Top Features Segregation Limit.
    :param criterion: `level/volume` according to which top entries will be picked up.
    :param verbose: Variable to Control Output to the console.
    :param prefix_name: Prefix Name to be added in front of each new encoded feature.
    :return: None/DataFrame.

    __Reference__
    *  Level : Will be considering up top `level` threshold entries for the particular feature, and rest as `BELOW`.
    *  Amount : Will be considering up the entries above the threshold for the particular feature, and rest as `BELOW`.
    """

    # Creating a Copy()
    train_set, test_set = TopLabeledEntries(train, test, feature_name, threshold, criterion, verbose = False,
                                            return_dataframe = True)

    # One-Hot Encoding the Top [Threshold] Features
    train_set = pd.get_dummies(train_set, columns = [feature_name], prefix = prefix_name)
    test_set = pd.get_dummies(test_set, columns = [feature_name], prefix = prefix_name)

    # Taking out the Encoded Features
    train_encoded_cols = train_set.filter(regex = f"{prefix_name}.*").columns
    test_encoded_cols = test_set.filter(regex = f"{prefix_name}.*").columns

    # Checking out Missing Feature(s)
    train_set_missing_cols = list(set(test_encoded_cols) - set(train_encoded_cols))
    test_set_missing_cols = list(set(train_encoded_cols) - set(test_encoded_cols))

    # Printing the Result if Verbose is True
    if verbose:
        print()
        print(f"Missing Features in Train : {train_set_missing_cols}")
        print(f"Missing Features in Test : {test_set_missing_cols}")
        print()

    # Adding up Missing Feature(s) in the Respective DataFrames [0-Padding]

    for i in range(len(train_set_missing_cols)):
        train_set[train_set_missing_cols[i]] = [0] * len(train)

    for i in range(len(test_set_missing_cols)):
        test_set[test_set_missing_cols[i]] = [0] * len(test)

    # Printing the Result if Verbose is True
    if verbose:

        print("*" * 15, "New Train Head", "*" * 15)
        print(train_set.head(1))
        print("*" * 15, "New Test Head", "*" * 15)
        print(test_set.head(1))

    # Printing out Operation Done
    print(f'`{feature_name}` Rare Label One-Hot Encoded!')

    # Returning the DataFrame
    return train_set, test_set




